import os
import typing
from pathlib import Path, PosixPath, WindowsPath


class BatchRelpath:
    """Manage the relative file path to given of paths from a starting
    point.
    """

    def __init__(self, paths: typing.List[Path], start: Path) -> None:
        """Initialize BatchRelPath.

        Args:
            paths (typing.List[Path]): A list of Path objects.
            start (Path): A Path object.
        """
        self.paths: typing.List[Path] = []
        self.start = start
        self.relpaths: typing.List[Path] = []

        # If the start contains a file, then split it from the path.
        if os.path.isfile(self.start.resolve()):
            head, tail = os.path.split(self.start.resolve())
            self.start = Path(head)

        self.add_paths(paths)

    def add_paths(self, paths: typing.List[Path]) -> None:
        """Add relative or absolute paths to the list of paths.

        Args:
            paths (typing.List[Path]): A list of Path objects.

        Raises:
            TypeError: Raised if any elements within the list are not
            Paths.
        """
        for p in paths:

            if type(p) != (PosixPath or WindowsPath):
                raise TypeError(
                    f"{p.name}: path must be of type PosixPath or WindowsPath"
                )
            else:

                if p.exists() is False:
                    raise IOError(f"{p.name}: invalid or cannot be found")
                else:
                    self.paths.append(p)

        self.__find_relpath()

    def remove_paths(self, paths: typing.List[Path]) -> None:
        """Remove relative or absolute paths from the list of paths.

        Args:
            paths (typing.List[Path]): A list of Path objects.
        """
        for path in paths:
            self.paths.remove(path)

        self.__find_relpath()

    def __find_relpath(self) -> None:
        """Find the relative file path to given paths from a starting
        point.
        """
        self.relpaths = []
        for p in self.paths:

            # Paths must be resolved to be placed in os.path.relpath().
            new_path = Path(os.path.relpath(p.resolve(), self.start.resolve()))

            self.relpaths.append(new_path)

    def __str__(self) -> str:
        """Print the relative file path to given paths from a starting
        point.
        """
        str_relpaths = []
        for p in self.relpaths:
            str_relpaths.append(str(p))

        return "\n".join(str_relpaths)
